<?php

// XArray.class.php - functions to assist when using arrays
//
// $Id: XArray.class.php 1424 2008-12-29 16:09:36Z tim $
//
// (c)2000-2004 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require_once('XString.class.php');

define('XARRAY_HTMLCONV_ON',false);	// Retain 'false' for backward compatibility
define('XARRAY_HTMLCONV_OFF',true);

define('XARRAY_ASSOC',true);// Retain 'true' for backward compatibility
define('XARRAY_LINEAR',false);
define('XARRAY_LINEAR_WITHVALS',false);

define('XARRAY_COMPARISON_LOOSE',1);
define('XARRAY_COMPARISON_STRICT',2);
define('XARRAY_COMPARISON_STRICTSTRING',3);

/**
 * This class contains useful functions for dealing with and manipulating 
 * arrays, and is primarily intended to be used uninstantiated. 
 *
 */
class XArray {

	/**
	 * Inserts a new element at the top of an associative array. It is safe with numeric keys, unlike PHP's array_merge().
	 *
	 * Unless I have missed something, an oversight in PHP is that there is no 
	 * way to easily add a new key/value pair to the start of an associative 
	 * array. This may be because the order of elements within associative 
	 * arrays often isn't important since you are accessing them using keys. 
	 * However, one common problem with Web programming is that you wish to 
	 * create an array dynamically (perhaps from a database) and then create a 
	 * HTML drop-down selector with the array. However, you also need to add a 
	 * sensible default to the top of the box (e.g. 'Select one...') whilst 
	 * preserving the order of the other elements.
	 * 
	 * Useful used in conjunction with XArray::CreateSelect.
	 * 
	 * Example:
	 * 
	 * $modem_array = array ('se56'  => 'SupraExpress 56e',
	 *                       'usr56' => 'USRobotics 56k',
	 *                       'winm'  => 'Internal Winmodem');
	 * 
	 * // This doesn't work - it creates a new element within $modem_array which 
	 * // contains the given array
	 * array_push ($modem_array,array('none'=>'Select one...'));
	 * 
	 * // This doesn't work either, for similar reasons
	 * $top_entry = array ('none'=>'Select one...');
	 * $modem_array = array_merge ( $top_entry, $modem_array);
	 * 
	 * // So, use XArray::addToHead
	 * $modem_array = XArray::addToHead('none','Select one...',$modem_array);
	 * 
	 * $modem_array now contains:
	 * 
	 * <option value='none'>Select one...</options>
	 * <option value='se56'>SupraExpress 56e</option>
	 * <option value='usr56' selected>USRobotics 56k</option>
	 * <option value='winm'>Internal Winmodem</option>
	 * 
	 * @param string $key_top  The key of the associative array to add
	 * @param string $value_top The value of the associative array to add
	 * @param array $existing_array  Existing array to add to
	 * @return array
	 * @see XArray::createSelect()
	 */
	function addToHead ($key_top, $value_top, $existing_array) {
		$new_array[$key_top] = $value_top;
		if (is_array($existing_array) && count($existing_array) > 0) {
			foreach ($existing_array as $key => $value) {
				$new_array[$key] = $value;
			}
		}
		return $new_array;
	}

	// 
	//		
	/**
	 * Creates the content of <select> containers
	 * 
	 * This function creates the contents of an HTML <select> container from an
	 * array. It's invaluable when working with web-based applications, where 
	 * you very frequently have the need to offer users a fixed set of choices 
	 * from which to make a selection. Often these choices are created from 
	 * entries in a database (which have been parsed into an array, for example 
	 * with DBTable::GetArray or similar), and frequently there will be the need
	 * to have a particular option pre-selected (especially in back-end 
	 * administration systems where database entries can be edited and the 
	 * current value of an entry must be refleced).
	 * 
	 * In the simplest case, you simply pass it an array. XArray::createSelect 
	 * will then assume the array is an associative array and will create 
	 * appropriate <option> lines. So, for example, if array is in the form 
	 * $key => $value, then the <option> lines will be of the form 
	 * <option value="$key">$value<option>.
	 * 
	 * If you specify the optional parameter current_selection, then the HTML 
	 * returned will contain an appropriate 'selected' term so that the select 
	 * box defaults to the requested option.
	 * 
	 * Using XArray::CreateSelect with the option is_associative parameter set 
	 * to FALSE will mean that the array is assumed to be linear, and the option
	 * values will be omitted.
	 * 
	 * Note that in no event does XArray::CreateSelect produce the containing 
	 * HTML <select> tags; it leaves those for you to generate since you will 
	 * need to customise them with names, classes etc. Rather, it produces only 
	 * the <option> tags to go inside.
	 * 
	 * Example 1:
	 * 
	 * $modem_array = array ('se56'  => 'SupraExpress 56e',
	 *                       'usr56' => 'USRobotics 56k',
	 *                       'winm'  => 'Internal Winmodem');
	 * 
	 * $html_associative = XArray::createSelect($modem_array);
	 * 
	 * $html_associative now contains:
	 * 
	 * <option value='se56'>SupraExpress 56e</option>
	 * <option value='usr56'>USRobotics 56k</option>
	 * <option value='winm'>Internal Winmodem</option>
	 * 
	 * Example 2 (using $modem_array from Example 1):
	 * $html_associative_selected = XArray::createSelect($modem_array,'usr56');
	 * 
	 * $html_associative_selected now contains:
	 * <option value='se56'>SupraExpress 56e</option>
	 * <option value='usr56' selected>USRobotics 56k</option>
	 * <option value='winm'>Internal Winmodem</option>
	 * 
	 * Example 3 (using $modem_array from Example 1)
	 * 
	 * $html_linear = XArray::CreateSelect($modem_array,'',false);
	 * 
	 * $html_linear now contains:
	 * 
	 * <option>SupraExpress 56e</option>
	 * <option>USRobotics 56k</option>
	 * <option>Internal Winmodem</option>
	 *
	 * @param array $array  List of options, in the form form_variable_name => option name to display
	 * @param string $selected  the form variable name of the currently-selected item
	 * @param const $ArrayType
	 * @param const $HTMLMode
	 * @param const $ComparisonType  An XARRAY_COMPARISON_* constant
	 * @return string HTML output
	 */
	function createSelect ($array, $selected = '', $ArrayType=XARRAY_ASSOC, $HTMLMode=XARRAY_HTMLCONV_ON, $ComparisonType=XARRAY_COMPARISON_LOOSE) {
		global $PHPOF_XHTML_COMPATIBILITY;
		if (!is_array($array)) trigger_error('XArray::createSelect() passed a non-array',E_USER_WARNING);
		$HTML = "\n";
		if ($PHPOF_XHTML_COMPATIBILITY) {
			$SelectedCode = " selected='selected'";
		} else {
			$SelectedCode = ' selected';
		}
		if (count($array) > 0) {
			foreach ($array as $item_ref => $item_desc) {
				$HTML .= "\t<option";
				if ($ArrayType == XARRAY_ASSOC) {
					$HTML .= " value='$item_ref'";
					$SearchItem = $item_ref;
				} else {
					$SearchItem = $item_desc;
					if ($ArrayType == XARRAY_LINEAR_WITHVALS) {
						$HTML .= " value='$item_desc'";
					}
				}
				if (is_array($selected)) {
					if (in_array($SearchItem,$selected)) $HTML .= $SelectedCode;
				} else {
					$IsSelected = false;
					switch ($ComparisonType) {
						case XARRAY_COMPARISON_LOOSE:
							$IsSelected = ($SearchItem == $selected);
							break;
						case XARRAY_COMPARISON_STRICT:
							$IsSelected = ($SearchItem === $selected);
							break;
						case XARRAY_COMPARISON_STRICTSTRING:
							settype($SearchItem,'string');
							settype($selected,'string');
							$IsSelected = ($SearchItem === $selected);
							break;
					}
					if ($IsSelected) $HTML .= $SelectedCode;
				}
				if ($HTMLMode == XARRAY_HTMLCONV_ON) {
					$HTML .= ">".XString::Output($item_desc)."</option>\n";
				} else {
					$HTML .= ">$item_desc</option>\n";
				}
			}
		}
		return $HTML;
	}

	/**
	 * Similar to XArray::createSelect() but creates radio buttons
	 *
	 * @param string $name
	 * @param array $array
	 * @param string $selected
	 * @param const $ArrayType
	 * @param const $HTMLMode
	 * @param string $onclick
	 * @param const $ComparisonType
	 * @return string HTML output
	 */
	function createRadioSelect($name, $array, $selected = '', $ArrayType=XARRAY_ASSOC, $HTMLMode=XARRAY_HTMLCONV_ON, $onclick='', $ComparisonType=XARRAY_COMPARISON_LOOSE) {
		global $PHPOF_XHTML_COMPATIBILITY;
		if (!is_array($array)) trigger_error('XArray::createRadioSelect() passed a non-array',E_USER_WARNING);
		$HTML = "\n";
		if ($PHPOF_XHTML_COMPATIBILITY) {
			$SelectedCode = " checked='checked'";
			$EE = ' /';
		} else {
			$SelectedCode = ' checked';
			$EE = '';
		}
		if (count($array) > 0) {
			foreach ($array as $item_ref => $item_desc) {
				$ID = $name.'_'.$item_ref;
				$ID = str_replace('[','__',$ID);
				$ID = str_replace(']','__',$ID);
				$HTML .= "\t<input type='radio' name='$name' id='$ID'";
				if (!empty($onclick)) $HTML .= " onclick=\"$onclick\"";
				if ($ArrayType == XARRAY_ASSOC) {
					$HTML .= " value='$item_ref'";
					$SearchItem = $item_ref;
				} else {
					$SearchItem = $item_desc;
					if ($ArrayType == XARRAY_LINEAR_WITHVALS) {
						$HTML .= " value='$item_desc'";
					}
				}
				
				$IsSelected = false;
				switch ($ComparisonType) {
					case XARRAY_COMPARISON_LOOSE:
						$IsSelected = ($SearchItem == $selected);
						break;
					case XARRAY_COMPARISON_STRICT:
						$IsSelected = ($SearchItem === $selected);
						break;
					case XARRAY_COMPARISON_STRICTSTRING:
						settype($SearchItem,'string');
						settype($selected,'string');
						$IsSelected = ($SearchItem === $selected);
						break;
				}
				if ($IsSelected) $HTML .= $SelectedCode;
				
				$HTML .= "$EE><label for='$ID'>";
				if ($HTMLMode == XARRAY_HTMLCONV_ON) {
					$HTML .= XString::Output($item_desc);
				} else {
					$HTML .= $item_desc;
				}
				$HTML .= "</label>\n";
			}
		}
		return $HTML;
	}
	
	function CreateHTMLColumns ($array, $columns, $col_prefix='', $col_suffix='') {
		$lines = count($array);
		$columnsleft = $columns;
		$arrayrow = 0;
		$html = "<table border='0' width='100%'><tr valign='top'>";
		for ($col = 0; $col < $columns; $col++) {
			$linespercol = ceil($lines/$columnsleft);
			if ($lines) $html .= "<td>\n".$col_prefix;
			for ($line = 1; $line <= $linespercol; $line++) {
				$html.= $array[$arrayrow];
				$lines--;
				$arrayrow++;
			}
			$columnsleft--;
			if ($lines) $html .= $col_suffix . "\n</td>";
		}
		$html .= "</tr>\n</table>";
		return $html;
	}

}